package zinger.util.recycling;

import java.util.*;

/**
 * Performs the same task as its superclass, but allows the ability to specify the maximum number
 * of cached instances.  Useful when we know the approximate range of usage and don't want too many
 * objects sitting cached in memory indefinitely after a surge.
 */
public class CappedObjectRecycler extends ObjectRecycler
{
    protected final int cap;

    public CappedObjectRecycler(ObjectGenerator generator, List instances, int cap)
    {
		super(generator, instances);
		this.cap = cap;
	}

    public CappedObjectRecycler(ObjectGenerator generator, int cap)
    {
		this(generator, null, cap);
    }

	/**
	 * Rejects the object if the instance cache exceeds specified size.
	 */
    public boolean recycleObject(final Object obj)
    {
		if(instances.size() < cap)
		{
			return super.recycleObject(obj);
		}
		return false;
    }
}