package zinger.util.recycling;

/**
 * An interface for a simple instance factory.  This is used by <code>ObjectRecycler</code>.
 * @see zinger.util.recycling.ObjectRecycler
 */
public interface ObjectGenerator
{
    /**
     * Creates a new instance.
     */
    public Object newObject();

    /**
     * Prepares the instance for use (reuse) according to the argument passed.
     * Since we can't absolutely be sure that the object being worked on is in valid state or
     * even that it was generated by this instance, we have a chance to refuse to prepare it
     * by returning <code>false</code>.
     * @return <code>true</code> if the preparation succeeded
     * <br><code>false</code> if there is something
     * wrong with the object passed
     * @throws java.lang.IllegalArgumentException if the argument passed for preparation is inappropriate
     */
    public boolean prepareObject(final Object obj, final Object arg) throws IllegalArgumentException;
}
